import numpy as np
from tensorflow import keras
from tensorflow.keras import layers
from sklearn.preprocessing import LabelBinarizer

###########################
## Simple Neural Network ##
###########################

# The neural network below operates on a number of numbers,
# and is trained to identify the one that is maximal.
# For instance: Given the numbers 3, 6, -2, the expected result
# would be 1, because number 6 is the largest, and it has
# the index number 1.
# This is not a complex task, and training a neural network
# to do it, is a (small) waste of electricity. But we can easily
# generate training data for this task, and it can be used to
# learn about neural networks.
# Note: The network below is more complex than it needs to be.

# VARIABLES
# The following variables control key parameters for the task

# The number of numbers in which we search for the maximum
# Length of (number) sequence
num_numbers = 5

# The number of epochs for training the network
# One epoch corresponds to one iteration over the entire
# training data set.
epochs = 100

# The number of instances after which we update the network
# weights
batch_size = 10

# The size of the training data set
training_data_size = 10000


# TRAINING DATA
# create a data set with random numbers
x_train = np.random.randn(training_data_size,num_numbers)

# Target value: What's the maximum (index) in each row?
# (0.1, 0.2, -0.2, 0.5, -3)
# -> (4)
y_train = np.array([(np.argmax(x)) for x in x_train])

# one-hot-encoding of target values
lb = LabelBinarizer()
y_train = lb.fit_transform(y_train)

# setup the model architecture
model = keras.Sequential()
model.add(layers.Input(shape=(num_numbers,)))
model.add(layers.Dense(10, activation="sigmoid"))
model.add(layers.Dense(num_numbers, activation="softmax"))

# compile it
model.compile(loss="categorical_crossentropy",
  optimizer="sgd",
  metrics=["accuracy"])

# train it (this line is the actual training)
model.fit(x_train, y_train, epochs=3, batch_size=batch_size)

# create a test data set
x_test = np.random.randn(100,num_numbers)
y_test = np.array([np.argmax(x) for x in x_test])
model.evaluate(x=x_test, y=lb.fit_transform(y_test))
